(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/CodGenCdrv.sml,v
   Comment: Generate a generic driver (for code generation purposes only!)
   Release: scope-0-15_final 

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   CodGenCdrv.sml,v 1.8 2003/07/08 15:45:01 wasowski Exp
*)

structure CodGenCdrv :CODGEN = struct

open Absyn CodGenC 

structure S  = Symbol
structure SE = SymbolEnv

infix 0 << <<< <<> <- (* some fun with C++ style IO *)
infix 3 --;

(* optional types *)

type options = bool * bool * bool * string option * string option

type aor = unit
type atr = unit

(* debug, smoke, VS compatibility, def file name, root of file names produced by main CODGEN (i.e C1) *)

val warn_generated = "/* This is a generated file! */\n/* Don't edit it manually.   */\n"

fun gen_vspreamble (m :(aor,atr) model) :codgen = 
	let val sys     = hd m
	    val eventno = SE.count (#evtypes sys)
	    val signals = List.exists (fn (na,(intr,ps))=>intr) 
				(SE.toList(#evtypes sys))
	    val sigmacr = if signals then DEFINE "SCOPE_SIGNALQUEUE" eoc 
					else eoc
	    val sysname = S.name (#name sys)
	    val sysdata = INCLUDE (sysname ^ "Data.h")
	    val sysactions = INCLUDE (sysname ^ "Action.h")

	in DEFINE "EVENTNO" (ig eventno) -- eol 
		-- sigmacr -- eol 
		-- sysdata -- eol
		-- sysactions -- eol
	end

(* SCOPE_SIGNALQUEUE, *)

fun compile (debug,smoke,vscompat,ofile,cg_filert) ifile (model :(aor,atr) model) :unit = 
	let val ofile = Option.getOpt (ofile,S.name(#name (hd model)) ^ "_main.c")
	    val cfile = TextIO.openOut ofile
	    val hfile = case cg_filert of NONE   => S.name (#name (hd model))
					| SOME s => s
	    val hincl = if vscompat then eoc else INCLUDE ( hfile ^ ".h" ) 
	    val root  = if vscompat then "CodGenVS_" else "CodGenC1_"
	in 
		cfile << warn_generated <- eol <- hincl <- eol;

		if vscompat then cfile <- gen_vspreamble model <- eol
		else cfile;

		if smoke then 
			cfile <- INCLUDESTD (root ^ "smokedrv.c") <- eol
		else if debug then 
			cfile <- INCLUDESTD (root ^ "testdrv.c") <- eol
		else (*release*) 
			cfile <- INCLUDESTD (root ^ "tinydrv.c") <- eol;

		TextIO.closeOut cfile
	end

end

