(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/acfg/acfg-vis-fn.sml,v
   Comment: ACFG visualizer - generates DOT graphs
   Release: scope-0-15_final

     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   acfg-vis-fn.sml,v 1.3 2003/05/06 11:13:45 wasowski Exp 
*)

functor ACFGVisFn (G :ACFG) :> ACFG_VIS 
		where type (''a,''b) acfg = (''a,''b) G.acfg = 
struct

  open G 
  structure B = bdd
  structure D = Dot

  val --> = D.-->
  infix 0 -->

  (* FIXME: rudimentary *)
  fun act_node (praction :''b -> string) (id :int) 
		(acfg :(''a,''b) acfg) : int * D.stmt = 
	case map praction (actions acfg) of
		  [] => (id+1, D.Node {id=Int.toString id, 
					attrs=[D.Shape D.Point]})
		| acts =>
			let val label= String.concat (
				foldr (fn (a,s) => a ::"\\n" :: s) 
				[Int.toString (length acts)," action(s)"] acts)
			in (id+1,D.Node {id=Int.toString id, 
					attrs=[D.Shape D.Box, D.Label label]})
			end
  
  (* FIXME: rudimentary *)
  fun test_node (prcond :''a -> string) (id :int) 
		(test :(''a,''b) test) :int * D.stmt =
	(id+1,D.Node {id=Int.toString id,
		      attrs= [D.Shape D.Diamond,D.Label (prcond (cond test))]})
	
  fun act_edges (id :int) (ts :int list) (ss :D.stmt list) : D.stmt list =
	let val a = Int.toString id
	in foldl (fn (t,e) => (a --> Int.toString t) [] ::e) ss ts end

  fun cond_edges (id :int) (lid :int) (hid :int) (ss :D.stmt list) 
		:D.stmt list = 
	let val a = Int.toString id
	in (a --> Int.toString lid) [D.Style D.Dotted]
		:: (a --> Int.toString hid) [] :: ss
	end
	
	
	
  fun draw (prcond :''a -> string) (praction :''b -> string)
		(acfg :(''a,''b) acfg) :Dot.dot =
	let 
	    fun draw_acfg acfg (id :int, ss :D.stmt list) 
				:int * D.stmt list = 
			let val (id1,node)  = act_node praction id acfg
			    val (id2,ts,ss) = foldl draw_test (id1,[],ss) 
						(tests acfg)

			in (id2, node::act_edges id ts ss) end 

            and draw_test (test,((id,idtests,ss) :int * int list * D.stmt list)) 
				:int * int list * D.stmt list = 
			let val (id1,node) = test_node prcond id test
			    val (lacfg,hacfg) = (lo test,hi test)
			    val (id2,ss1) = draw_acfg hacfg (id1,ss)
			    val (id3,ss2) = draw_acfg lacfg (id2,ss1)
			in (id3,id::idtests,node::cond_edges id id2 id1 ss2) end
	    val root = []	
	    val globals = [Dot.NodeAttr [Dot.FontName "Helvetica", Dot.FontSize 10]]
	    val g = { id ="acfg", stmts = globals @ #2 (draw_acfg acfg (0,root)), 
		cluster=false } 
	in [ (false,true,g) ] end
end
(*
	The convention for type variables:
	 ''a denotes type of conditions,
	 ''b denotes type of actions,
	 ''c denotes type of state of computation of heuristics 
*)

