(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/test/src/tInterprLang.sml,v
   Comment: Language for specifying testcases for tInterpr
   Release: scope-0-15_final

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   NOTES:

   The language is build as a set of combinators. Valid specification
   translates itself automatically to an SML datastructure representing 
   it.

   tInterprLang.sml,v 1.7 2002/12/10 16:16:58 wasowski Exp
*)
  
  open Test Absyn;

  (* abstract syntax for specifying test cases *)

  type symbol = Symbol.symbol
  val  symbol = Symbol.symbol
  val  name   = Symbol.name

  type einst    = Interpr.einst
  type oinst    = Interpr.oinst

  fun isconf (conf :confspec) = conf <> [""]

  type stepspec = { input     : einst,
		    endconf   : confspec option,	(* NONE means no change *)
		    outputs   : oinst list };
						(* NONE means don't test *)

  datatype step = INIT of stepspec list  (* initialize state AFTER!*)
		| CONT of stepspec list  (* return resulting state *)

  type testcase = { file    : string,
		    iniconf : (confspec option  * oinst list) list,
		    steps   : step list
		  };



  (* test language combinators *)		  

  datatype then_token = THEN

  val END = []

  val NO_CHANGE = [""]; (* a hack: "" is an illegal state name *)
  val DONT_TEST = ([""],[]); (* a hack: "" is an illegal state name *)

  fun IF (ev :string) (params :value list) THEN
	: confspec option -> oinst list -> stepspec =
	fn conf => fn ois => {input=(symbol ev,params), 
			      endconf=conf, 
			      outputs=ois}

  fun STATE (f :confspec option -> oinst list -> stepspec, 
             conf :confspec) :oinst list -> stepspec = 
	     if isconf conf then f (SOME conf) else f NONE

  fun OUT (f :oinst list -> stepspec,
           outputs :oinst list) :stepspec = f outputs

  fun !! (output :string) (params :value list) :oinst = 
		(symbol output,params)

  val || = op::

  fun CONTINUE (s :stepspec list, ss :step list) :step list = CONT s :: ss
  
  fun RESTART (s :stepspec list, ss :step list) :step list = INIT s :: ss

  local 
	fun deempty (conf,os) = if isconf conf then (SOME conf,os) 
				else (NONE,os)
  in
        fun INITIAL (filename :string, inis : (confspec * oinst list) list) 
					:step list -> testcase = 
		fn sl => {file    = filename, 
			  iniconf = map deempty inis, 
			  steps   = sl}
  end


  fun STEPS (f :step list -> testcase, steps :step list) :testcase = f steps

  val NEXT = op::;
 
  (* fixities needed to give specification reasonable look: *)

  infix  6 OUT;
  infix  6 STATE;
  infixr 5 INITIAL;
  infixr 5 ||;
  infixr 4 RESTART;
  infixr 4 CONTINUE;
  infixr 3 STEPS;
  infixr 3 NEXT;
