(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/util/UList.sml,v
   Comment: Modelling notupdatable memory arrays of unique elements with lists
   Release: scope-0-15_final

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  UList.sml,v 1.4 2003/07/24 15:36:24 wasowski Exp
*)
  exception NotFound

  type ''a ulist = (''a * int) list * (''a -> int)

(* first pair element : a list of pairs. Each pair represents an ulist
   element and the address of the *next* position after it, or length of the
   list otherwise.
   second pair element: a measure to decide size of list elements *)

  fun initialize sizer = ([],sizer)

  fun size [] :int = 0
    | size ((x,s)::xs) = s

  fun length ((lst,sizer) :''a ulist) :int = size lst

  fun alloc (e :''a, (lst,sizer) :''a ulist) : int * ''a ulist = 
	let fun f [] = (size lst,((e,size lst+sizer e)::lst,sizer))
              | f ((x,s)::xs) = if e = x then (size xs,(lst,sizer)) else f xs 
	in f lst end

  fun find (e :''a, (lst,size) :''a ulist) : int =
	let fun f []       = raise NotFound
	      | f [he]     = if #1 he = e then 0 else raise NotFound
	      | f (he::ne::ta) = if #1 he = e then #2 ne else f (ne::ta)
	in f lst end

  fun toList (xs :''a ulist) :''a list = map #1 (#1 xs)

  fun null ([],size) = true
    | null _         = false

  
fun foldl f e (ul :''a ulist) = 
	let fun fold e []      = e
	      | fold e (x::xs) = fold (f(#1 x,e)) xs
	in fold e (#1 ul) end

fun foldr f e (ul :''a ulist) = 
	let fun fold e []      = e
	      | fold e (x::xs) = f(#1 x,fold e xs)
	in fold e (#1 ul) end

fun mapl f (l :''a ulist) = map (fn(a,_)=>f a) (#1 l)
